@@ -25,9 +25,11 @@ import warnings
 
 import abc
 import netaddr
+import rfc3986
 import six
 
 
+@six.add_metaclass(abc.ABCMeta)
 class ConfigType(object):
     def __init__(self, type_name='unknown type'):
         self.type_name = type_name
@@ -165,7 +167,9 @@ class String(ConfigType):
     def __eq__(self, other):
         return (
             (self.__class__ == other.__class__) and
-            (self.choices == other.choices) and
+            (set(self.choices) == set(other.choices) if
+             self.choices and other.choices else
+             self.choices == other.choices) and
             (self.quotes == other.quotes) and
             (self.regex == other.regex)
         )
@@ -175,6 +179,8 @@ class String(ConfigType):
 
 
 class MultiString(String):
+    """Multi-valued string."""
+
     def __init__(self, type_name='multi valued'):
         super(MultiString, self).__init__(type_name=type_name)
 
@@ -237,101 +243,148 @@ class Boolean(ConfigType):
         return str(value).lower()
 
 
-class Integer(ConfigType):
+class Number(ConfigType):
 
-    """Integer type.
+    """Number class, base for Integer and Float.
 
-    Converts value to an integer optionally doing range checking.
-    If value is whitespace or empty string will return None.
-
-    :param min: Optional check that value is greater than or equal to min
-    :param max: Optional check that value is less than or equal to max
+    :param min: Optional check that value is greater than or equal to min.
+                Mutually exclusive with 'choices'.
+    :param max: Optional check that value is less than or equal to max.
+                Mutually exclusive with 'choices'.
     :param type_name: Type name to be used in the sample config file.
+    :param choices: Optional sequence of valid values. Mutually exclusive
+                    with 'min/max'.
+    :param num_type: the type of number used for casting (i.e int, float)
 
-    .. versionchanged:: 2.4
-       The class now honors zero for *min* and *max* parameters.
-
-    .. versionchanged:: 2.7
-       Added *type_name* parameter.
+    .. versionadded:: 3.14
     """
 
-    def __init__(self, min=None, max=None, type_name='integer value'):
-        super(Integer, self).__init__(type_name=type_name)
+    def __init__(self, num_type, type_name,
+                 min=None, max=None, choices=None):
+        super(Number, self).__init__(type_name=type_name)
+
+        # Validate the choices and limits
+        if choices is not None:
+            if min is not None or max is not None:
+                raise ValueError("'choices' and 'min/max' cannot both be "
+                                 "specified")
+        else:
+            if min is not None and max is not None and max < min:
+                raise ValueError('Max value is less than min value')
+
         self.min = min
         self.max = max
-        if min is not None and max is not None and max < min:
-            raise ValueError('Max value is less than min value')
+        self.choices = choices
+        self.num_type = num_type
 
     def __call__(self, value):
-        if not isinstance(value, int):
+        if not isinstance(value, self.num_type):
             s = str(value).strip()
             if s == '':
                 value = None
             else:
-                value = int(value)
+                value = self.num_type(value)
 
         if value is not None:
-            self._check_range(value)
+            if self.choices is not None:
+                self._check_choices(value)
+            else:
+                self._check_range(value)
 
         return value
 
+    def _check_choices(self, value):
+        if value in self.choices:
+            return
+        else:
+            raise ValueError('Valid values are %r, but found %g' % (
+                             self.choices, value))
+
     def _check_range(self, value):
         if self.min is not None and value < self.min:
-            raise ValueError('Should be greater than or equal to %d' %
+            raise ValueError('Should be greater than or equal to %g' %
                              self.min)
         if self.max is not None and value > self.max:
-            raise ValueError('Should be less than or equal to %d' % self.max)
+            raise ValueError('Should be less than or equal to %g' % self.max)
 
     def __repr__(self):
         props = []
-        if self.min is not None:
-            props.append('min=%d' % self.min)
-        if self.max is not None:
-            props.append('max=%d' % self.max)
+        if self.choices is not None:
+            props.append("choices=%r" % (self.choices,))
+        else:
+            if self.min is not None:
+                props.append('min=%g' % self.min)
+            if self.max is not None:
+                props.append('max=%g' % self.max)
 
         if props:
-            return 'Integer(%s)' % ', '.join(props)
-        return 'Integer'
+            return self.__class__.__name__ + '(%s)' % ', '.join(props)
+        return self.__class__.__name__
 
     def __eq__(self, other):
         return (
             (self.__class__ == other.__class__) and
             (self.min == other.min) and
-            (self.max == other.max)
+            (self.max == other.max) and
+            (set(self.choices) == set(other.choices) if
+             self.choices and other.choices else
+             self.choices == other.choices)
         )
 
     def _formatter(self, value):
         return str(value)
 
 
-class Float(ConfigType):
+class Integer(Number):
+
+    """Integer type.
+
+    Converts value to an integer optionally doing range checking.
+    If value is whitespace or empty string will return None.
+
+    :param min: Optional check that value is greater than or equal to min.
+                Mutually exclusive with 'choices'.
+    :param max: Optional check that value is less than or equal to max.
+                Mutually exclusive with 'choices'.
+    :param type_name: Type name to be used in the sample config file.
+    :param choices: Optional sequence of valid values. Mutually exclusive
+                    with 'min/max'.
+
+    .. versionchanged:: 2.4
+       The class now honors zero for *min* and *max* parameters.
+
+    .. versionchanged:: 2.7
+       Added *type_name* parameter.
+
+    .. versionchanged:: 3.2
+       Added *choices* parameter.
+    """
+
+    def __init__(self, min=None, max=None, type_name='integer value',
+                 choices=None):
+        super(Integer, self).__init__(int, type_name, min=min, max=max,
+                                      choices=choices)
+
+
+class Float(Number):
 
     """Float type.
 
     :param type_name: Type name to be used in the sample config file.
+    :param min: Optional check that value is greater than or equal to min.
+    :param max: Optional check that value is less than or equal to max.
 
     .. versionchanged:: 2.7
 
        Added *type_name* parameter.
+
+    .. versionchanged:: 3.14
+
+       Added *min* and *max* parameters.
     """
 
-    def __init__(self, type_name='floating point value'):
-        super(Float, self).__init__(type_name=type_name)
-
-    def __call__(self, value):
-        if isinstance(value, float):
-            return value
-
-        return float(value)
-
-    def __repr__(self):
-        return 'Float'
-
-    def __eq__(self, other):
-        return self.__class__ == other.__class__
-
-    def _formatter(self, value):
-        return str(value)
+    def __init__(self, min=None, max=None, type_name='floating point value'):
+        super(Float, self).__init__(float, type_name, min=min, max=max)
 
 
 class List(ConfigType):
@@ -413,6 +466,8 @@ class List(ConfigType):
         )
 
     def _formatter(self, value):
+        if isinstance(value, six.string_types):
+            return value
         return ','.join(value)
 
 
@@ -529,7 +584,7 @@ class IPAddress(ConfigType):
        Added *type_name* parameter.
     """
 
-    def __init__(self, version=None, type_name='ip address value'):
+    def __init__(self, version=None, type_name='IP address value'):
         super(IPAddress, self).__init__(type_name=type_name)
         version_checkers = {
             None: self._check_both_versions,
@@ -569,3 +624,96 @@ class IPAddress(ConfigType):
 
     def _formatter(self, value):
         return value
+
+
+class Hostname(ConfigType):
+    """Hostname type.
+
+    A hostname refers to a valid DNS or hostname. It must not be longer than
+    253 characters, have a segment greater than 63 characters, nor start or
+    end with a hyphen.
+
+    :param type_name: Type name to be used in the sample config file.
+
+    """
+
+    def __init__(self, type_name='hostname value'):
+        super(Hostname, self).__init__(type_name=type_name)
+
+    def __call__(self, value):
+        """Check hostname is valid.
+
+        Ensures that each segment
+        - Contains at least one character and a maximum of 63 characters
+        - Consists only of allowed characters: letters (A-Z and a-z),
+          digits (0-9), and hyphen (-)
+        - Does not begin or end with a hyphen
+        - maximum total length of 253 characters
+
+        For more details , please see: http://tools.ietf.org/html/rfc1035
+        """
+
+        if len(value) == 0:
+            raise ValueError("Cannot have an empty hostname")
+        if len(value) > 253:
+            raise ValueError("hostname is greater than 253 characters: %s"
+                             % value)
+        if value.endswith("."):
+            value = value[:-1]
+        allowed = re.compile("(?!-)[A-Z0-9-]{1,63}(?<!-)$", re.IGNORECASE)
+        if any((not allowed.match(x)) for x in value.split(".")):
+            raise ValueError("%s is an invalid hostname" % value)
+        return value
+
+    def __repr__(self):
+        return 'Hostname'
+
+    def __eq__(self, other):
+        return self.__class__ == other.__class__
+
+    def _formatter(self, value):
+        return value
+
+
+class URI(ConfigType):
+
+    """URI type
+
+    Represents URI. Value will be validated as RFC 3986.
+
+    :param type_name: Type name to be used in the sample config file.
+    :param max_length: Optional integer. If a positive value is specified,
+                       a maximum length of an option value must be less than
+                       or equal to this parameter. Otherwise no length check
+                       will be done.
+
+    .. versionchanged:: 3.14
+       Added *max_length* parameter.
+    """
+
+    def __init__(self, max_length=None, type_name='uri value'):
+        super(URI, self).__init__(type_name=type_name)
+        self.max_length = max_length
+
+    def __call__(self, value):
+        if not rfc3986.is_valid_uri(value, require_scheme=True,
+                                    require_authority=True):
+            raise ValueError('invalid URI: %r' % value)
+
+        if self.max_length is not None and len(value) > self.max_length:
+            raise ValueError("Value '%s' exceeds maximum length %d" %
+                             (value, self.max_length))
+        self.value = value
+        return value
+
+    def __repr__(self):
+        return 'URI'
+
+    def __eq__(self, other):
+        return (
+            (self.__class__ == other.__class__) and
+            (self.value == other.value)
+        )
+
+    def _formatter(self, value):
+        return value
